<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2006 Reinhard Scheck aka gandalf                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 */

/* display_nectar_items		display the list of all items related to a single nectar
 * @arg $nectar_id				id of the nectar
 */
function display_nectar_items($nectar_id) {
	global $colors, $graph_timespans;
	global $item_types, $alignment;

	$items = db_fetch_assoc("(SELECT " .
					"plugin_nectar_items.id, " .
					"plugin_nectar_items.sequence, " .
					"plugin_nectar_items.item_type, " .
					"plugin_nectar_items.host_id, " .
					"host.description, " .
					"plugin_nectar_items.local_graph_id, " .
					"graph_templates_graph.title_cache, " .
					"plugin_nectar_items.timespan, " .
					"plugin_nectar_items.item_text, " .
					"plugin_nectar_items.align, " .
					"plugin_nectar_items.font_size " .
					"FROM plugin_nectar_items " .
					"LEFT JOIN host ON (plugin_nectar_items.host_id = host.id) " .
					"LEFT JOIN graph_templates_graph ON (plugin_nectar_items.local_graph_id=graph_templates_graph.local_graph_id) " .
					"WHERE report_id=" . $nectar_id .
					" AND item_type=" . NECTAR_ITEM_GRAPH .
					") UNION (" .
					"SELECT " .
					"plugin_nectar_items.id, " .
					"plugin_nectar_items.sequence, " .
					"plugin_nectar_items.item_type, " .
					"plugin_nectar_items.host_id, " .
					"'', " .
					"plugin_nectar_items.local_graph_id, " .
					"'', " .
					"plugin_nectar_items.timespan, " .
					"plugin_nectar_items.item_text, " .
					"plugin_nectar_items.align, " .
					"plugin_nectar_items.font_size " .
					"FROM plugin_nectar_items " .
					"WHERE report_id=" . $nectar_id .
					" AND item_type=" . NECTAR_ITEM_TEXT .
					") ORDER BY sequence");

	html_start_box("<strong>Report Items</strong>", "100%", $colors["header"], "3", "center", htmlspecialchars("nectar.php?action=item_edit&id=" . $nectar_id));

	print "<tr bgcolor='#" . $colors["header_panel"] . "'>";
	DrawMatrixHeaderItem("Item",$colors["header_text"],1);
	DrawMatrixHeaderItem("Sequence",$colors["header_text"],1);
	DrawMatrixHeaderItem("Type",$colors["header_text"],1);
	DrawMatrixHeaderItem("Host",$colors["header_text"],1);
	DrawMatrixHeaderItem("Graph",$colors["header_text"],1);
	DrawMatrixHeaderItem("Timespan",$colors["header_text"],1);
	DrawMatrixHeaderItem("Alignment",$colors["header_text"],1);
	DrawMatrixHeaderItem("Text",$colors["header_text"],1);
	DrawMatrixHeaderItem("Font Size",$colors["header_text"],1);
	DrawMatrixHeaderItem("&nbsp;",$colors["header_text"],2);
	print "</tr>";

	$i = 0;
	if (sizeof($items) > 0) {
		foreach ($items as $item) {
			#print "<pre>"; print_r($item); print "<pre>";

			form_alternate_row_color($colors["alternate"],$colors["light"],$i); $i++;
			$form_data = '<td><a class="linkEditMain" href="' . htmlspecialchars("nectar.php?action=item_edit&id=" . $nectar_id. "&item_id=" . $item["id"]) . '">Item#' . $i . '</a></td>';
			$form_data .= '<td>' . 	$item["sequence"] . '</td>';
			$form_data .= '<td>' . 	$item_types{$item["item_type"]} . '</td>';
			$form_data .= '<td>' . 	(($item["host_id"] == NECTAR_HOST_NONE) ? 'None' : $item["description"]) . '</td>';
			$form_data .= '<td>' . 	(($item["local_graph_id"] == 0) ? 'None' : $item["title_cache"]) . '</td>';
			$form_data .= '<td>' . 	(($item["timespan"] == 0) ? 'None' : $graph_timespans{$item["timespan"]}) . '</td>';
			$form_data .= '<td>' . 	(($item["align"] == 0) ? 'None' : $alignment{$item["align"]}) . '</td>';
			$form_data .= '<td>' . 	(($item["item_text"] == '') ? 'None' : $item["item_text"]) . '</td>';
			$form_data .= '<td>' . 	(($item["font_size"] == 0) ? 'None' : $item["font_size"]) . '</td>';
			$form_data .= '<td><a href="' . htmlspecialchars('nectar.php?action=item_movedown&item_id=' . $item["id"] . '&id=' . $nectar_id) .
							'"><img src="../../images/move_down.gif" border="0" alt="Move Down"></a>' .
							'<a	href="' . htmlspecialchars('nectar.php?action=item_moveup&item_id=' . $item["id"] .	'&id=' . $nectar_id) .
							'"><img src="../../images/move_up.gif" border="0" alt="Move Up"></a>' . '</td>';
			$form_data .= '<td align="right"><a href="' . htmlspecialchars('nectar.php?action=item_remove&item_id=' . $item["id"] .	'&id=' . $nectar_id) .
							'"><img src="../../images/delete_icon.gif" border="0" width="10" height="10" alt="Delete"></a>' . '</td></tr>';
			print $form_data;
		}
	} else {
		print "<tr><td><em>No Report Items</em></td></tr>\n";
	}
	html_end_box();
}

/* duplicate_nectar		duplicates a report and all items
 * @arg $_id				id of the report
 * @arg $_title				title of the new report
 */
function duplicate_nectar($_id, $_title) {
	global $fields_nectar_edit;

	$nectar = db_fetch_row("SELECT * FROM plugin_nectar WHERE id=$_id");
	$nectar_items = db_fetch_assoc("SELECT * FROM plugin_nectar_items WHERE report_id=$_id");

	$save = array();
	reset($fields_nectar_edit);
	while (list($field, $array) = each($fields_nectar_edit)) {
		if (!ereg("^hidden", $array["method"])) {
			$save[$field] = $nectar[$field];
		}
	}

	/* substitute the title variable */
	$save["name"] = str_replace("<name>", $nectar["name"], $_title);
	/* create new rule */
	$save["enabled"] = "";	# no new rule accidentally taking action immediately
	$save["id"] = 0;
	$nectar_id = sql_save($save, "plugin_nectar");

	/* create new rule items */
	if (sizeof($nectar_items) > 0) {
		foreach ($nectar_items as $nectar_item) {
			$save = $nectar_item;
			$save["id"] = 0;
			$save["report_id"] = $nectar_id;
			$nectar_item_id = sql_save($save, "plugin_nectar_items");
		}
	}
}


/* nectar_date_time_format		fetches the date/time formatting information for current user
 *
 */
function nectar_date_time_format() {
	global $config;

	$graph_date = "";

	/* setup date format */
	$date_fmt = read_graph_config_option("default_date_format");
	$datechar = read_graph_config_option("default_datechar");

	switch ($datechar) {
		case GDC_HYPHEN: 	$datechar = "-"; break;
		case GDC_SLASH: 	$datechar = "/"; break;
		case GDC_DOT:	 	$datechar = "."; break;
	}

	switch ($date_fmt) {
		case GD_MO_D_Y:
			$graph_date = "m" . $datechar . "d" . $datechar . "Y H:i:s";
			break;
		case GD_MN_D_Y:
			$graph_date = "M" . $datechar . "d" . $datechar . "Y H:i:s";
			break;
		case GD_D_MO_Y:
			$graph_date = "d" . $datechar . "m" . $datechar . "Y H:i:s";
			break;
		case GD_D_MN_Y:
			$graph_date = "d" . $datechar . "M" . $datechar . "Y H:i:s";
			break;
		case GD_Y_MO_D:
			$graph_date = "Y" . $datechar . "m" . $datechar . "d H:i:s";
			break;
		case GD_Y_MN_D:
			$graph_date = "Y" . $datechar . "M" . $datechar . "d H:i:s";
			break;
	}
	return $graph_date;
}

/* nectar_interval_start		computes the next start time for the given set of parameters
 * @arg $interval				given interval
 * @arg $count					given repeat count
 * @arg $offset					offset in seconds to be added to the new start time
 * @arg $timestamp				current start time for nectar
 * returns						new timestamp
 */
function nectar_interval_start($interval, $count, $offset, $timestamp) {
	global $nectar_interval;

	switch ($interval) {
		case NECTAR_SCHED_INTVL_DAY:
			# add $count days to current mailtime
			$ts = utime_add($timestamp,0,0,$count,0,0,$offset);
			break;
			# add $count weeks = 7*$count days to current mailtime
		case NECTAR_SCHED_INTVL_WEEK:
			$ts = utime_add($timestamp,0,0,7*$count,0,0,$offset);
			break;
		case NECTAR_SCHED_INTVL_MONTH_DAY:
			# add $count months to current mailtime
			$ts = utime_add($timestamp,0,$count,0,0,0,$offset);
			break;
		case NECTAR_SCHED_INTVL_MONTH_WEEKDAY:
			# add $count months to current mailtime, but if this is the nth weekday, it must be the same nth weekday in the new month
			# e.g. if this is currently the 3rd Monday of current month
			# ist must be the 3rd Monday of the new month as well
			$weekday = date('l', $timestamp);
			$day_of_month = date('j', $timestamp);
			$nth_weekday = ceil($day_of_month/7);

			$date_str = "+" . $count . " months";
			$month_base = strtotime($date_str, $timestamp);
			$new_month = mktime(date("H", $month_base), date("i", $month_base), date("s", $month_base), date("m", $month_base), 1, date("Y", $month_base));

			$date_str = "+" . ($nth_weekday -1) . " week " . $weekday;
			$base = strtotime($date_str, $new_month);
			$ts = mktime(date("H", $month_base), date("i", $month_base), date("s", $month_base), date("m", $base), date("d", $base), date("Y", $base));
			break;
		case NECTAR_SCHED_INTVL_YEAR:
			# add $count years to current mailtime
			$ts = utime_add($timestamp,$count,0,0,0,0,$offset);
			break;
		default:
			$ts = 0;
			break;
	}
	return $ts;
}

/* utime_add		add offsets to given timestamp
 * @arg $timestamp	base timestamp
 * @arg $yr			offset in years
 * @arg $mon		offset in months
 * @arg $day		offset in days
 * @arg $hr			offset in hours
 * @arg $min		offset in minutes
 * @arg $sec		offset in seconds
 */
function utime_add($timestamp, $yr=0, $mon=0, $day=0, $hr=0, $min=0, $sec=0) {
	$dt = localtime($timestamp, true);
	$unixnewtime = mktime(
	$dt['tm_hour']+$hr, $dt['tm_min']+$min, $dt['tm_sec']+$sec,
	$dt['tm_mon']+1+$mon, $dt['tm_mday']+$day, $dt['tm_year']+1900+$yr);
	return $unixnewtime;
}

/* nectar_log - logs a string to Cacti's log file or optionally to the browser
 @arg $string - the string to append to the log file
 @arg $output - (bool) whether to output the log line to the browser using pring() or not
 @arg $environ - (string) tell's from where the script was called from */
function nectar_log($string, $output = false, $environ="NECTAR", $level=POLLER_VERBOSITY_NONE) {
	# if current verbosity >= level of current message, print it
	if (NECTAR_DEBUG >= $level) {
		cacti_log($string, $output, $environ);
	}
}

/* generate_nectar		create the complete mail for a single report and send it
 * @arg $nectar				complete row of plugin_nectar table for the report to work upon
 * @arg $force				when forced, lastsent time will not be entered (e.g. Send Now)
 */
function generate_nectar($nectar, $force = false) {
	global $config, $alignment;
	include_once($config["base_path"] . "/lib/time.php");
	include_once($config["base_path"] . "/lib/rrd.php");

	nectar_log("generate_nectar, report_id: " . $nectar["id"], false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);

	$message = "<h3><center>" . $nectar['name'] . '</center></h3><ul>';
	# this function should be called only at the appropriate targeted time when in batch mode
	# but for preview mode we can't use the targeted time
	# so let's use time()
	$time = time();
	# get config option for first-day-of-the-week
	$first_weekdayid = read_graph_config_option("first_weekdayid");

	$nectar_items = db_fetch_assoc("SELECT * from plugin_nectar_items WHERE report_id=". $nectar['id'] . ' ORDER BY sequence');

	nectar_log("generate_nectar, items found: " . sizeof($nectar_items), false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);

	if (sizeof($nectar_items)) {
		foreach($nectar_items as $item) {
			nectar_log("generate_nectar, item_id: " . $item["id"] . " local_graph_id: " . $item["local_graph_id"], false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);
			if ($item['item_type'] == NECTAR_ITEM_GRAPH) {
				$timespan = array();
				# get start/end time-since-epoch for actual time (now()) and given current-session-timespan
				get_timespan($timespan, $time, $item["timespan"], $first_weekdayid);

				# provide parameters for rrdtool graph
				$graph_data_array = array(
					'graph_start'	=> $timespan["begin_now"],
					'graph_end'	 	=> $timespan["end_now"],
#					'print_source'	=> 'yes',
					'output_flag'	=> RRDTOOL_OUTPUT_STDOUT,
				);
				# provide parameters for mailing
				nectar_log("generate_nectar, item_id: " . $item["id"] . " local_graph_id: " . $item["local_graph_id"], false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);
				if ($nectar["attachment_type"] == NECTAR_TYPE_PNG) {
					$mail_data_array[] = array(
  						'attachment'	=> rrdtool_function_graph($item['local_graph_id'], "", $graph_data_array),
						'filename'	 	=> $item['local_graph_id'] . ".png",
						'mime_type'		=> 'image/png',
						'inline'		=> 'inline',
					);
				} elseif($nectar["attachment_type"] == NECTAR_TYPE_JPG) {
					$mail_data_array[] = array(
  						'attachment'	=> png2jpeg(rrdtool_function_graph($item['local_graph_id'], "", $graph_data_array)),
						'filename'	 	=> $item['local_graph_id'] . ".jpg",
						'mime_type'		=> 'image/png',
						'inline'		=> 'inline',
					);
				} elseif($nectar["attachment_type"] == NECTAR_TYPE_GIF) {
					$mail_data_array[] = array(
  						'attachment'	=> png2gif(rrdtool_function_graph($item['local_graph_id'], "", $graph_data_array)),
						'filename'	 	=> $item['local_graph_id'] . ".gif",
						'mime_type'		=> 'image/gif',
						'inline'		=> 'inline',
					);
				}
			} elseif ($item['item_type'] == NECTAR_ITEM_TEXT) {
				$message .= "<p align='" . $alignment[$item["align"]] . "' style='font-size: " . $item["font_size"] . "pt;'>" . $item["item_text"] . "</p>";
			}
		}
		$error = mg_send_mail($nectar['email'], $nectar['from_email'], $nectar['name'], $message, $mail_data_array);
		if (strlen($error)) {
			cacti_log($error, true, 'NECTAR');
		} else {
			if (!$force) {
				$next = nectar_interval_start($nectar["intrvl"], $nectar["count"], $nectar["offset"], $nectar['mailtime']);
				$next = floor($next / read_config_option("poller_interval")) * read_config_option("poller_interval");
				$sql = "UPDATE plugin_nectar SET mailtime = $next, lastsent=".time()." WHERE id = " . $nectar['id'];
				nectar_log("generate_nectar, update sql: " . $sql, false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);
				db_execute($sql);
			}
		}
	}
}

/* nectar_preview		print report to html for online verification
 * @arg $nectar			complete row of plugin_nectar table for the report to work upon
 */
function nectar_preview ($nectar_id) {
	global $config, $colors;
	global $alignment;
	include_once($config["base_path"] . "/lib/time.php");

	$nectar = db_fetch_row("select * from plugin_nectar where id = $nectar_id");
	$nectar_items = db_fetch_assoc("SELECT * from plugin_nectar_items WHERE report_id=". $nectar['id'] . ' ORDER BY sequence');

	nectar_log("generate_nectar, items found: " . sizeof($nectar_items), false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);

	if (sizeof($nectar_items)) {
		print '<br><br>';
		html_start_box("", "80%", $colors["header"], "1", "center", "");
		print "<table bgcolor='#FFFFFF' width='100%'><tr><td>";
		print "<h3><center>" . $nectar['name'] . '</center></h3><br><br><ul>';
		# this function should be called only at the appropriate targeted time when in batch mode
		# but for preview mode we can't use the targeted time
		# so let's use time()
		$time = time();
		# get config option for first-day-of-the-week
		$first_weekdayid = read_graph_config_option("first_weekdayid");

		foreach($nectar_items as $item) {
			nectar_log("generate_nectar, item_id: " . $item["id"] . " local_graph_id: " . $item["local_graph_id"], false, "NECTAR TRACE", POLLER_VERBOSITY_MEDIUM);
			if ($item['item_type'] == NECTAR_ITEM_GRAPH) {
				$timespan = array();
				# get start/end time-since-epoch for actual time (now()) and given current-session-timespan
				get_timespan($timespan, $time, $item["timespan"], $first_weekdayid);

				print "<p align='" . $alignment[$item["align"]] . "'>";
				print "<img src='../../graph_image.php?local_graph_id=" . $item['local_graph_id'] . "&graph_start=" . $timespan["begin_now"] . "&graph_end=" . $timespan["end_now"] ."'><br><br></p>";
			} elseif ($item['item_type'] == NECTAR_ITEM_TEXT) {
				print "<p align='" . $alignment[$item["align"]] . "' style='font-size: " . $item["font_size"] . "pt;'>" . $item["item_text"] . "</p>";
			}
		}
		print "</ul></td></tr></table>";
		html_end_box();
	}
}

/* This function is stolen from SETTINGS
 * and modified in a way, that rrdtool graph is kept outside this send_mail function
 * Using this function with current MAIL servers using extended authentication throws errors.
 */
function mg_send_mail($to, $from, $subject, $message, $data_array = '', $headers = '') {
	global $config;
	include_once($config["base_path"] . "/plugins/settings/include/mailer.php");

        /* avoid sending mails from hot standby system */
        $result = array();
        $cactiip = "10.225.64.27";
        $cmd = "/sbin/ifconfig |grep '10.225.64.27'";
        $res = exec($cmd, $result);
        if (!$res) {
                cacti_log("Sending EMail was suppressed due to not running on the active CACTI system", true, "NECTAR");
                return;
        }
        cacti_log("Sending Email to: '" . $to . "' - '" . $subject . "'", true, "NECTAR");

	$message = str_replace('<SUBJECT>', $subject, $message);
	$message = str_replace('<TO>', $to, $message);
	$message = str_replace('<FROM>', $from, $message);

	$how = read_config_option("settings_how");
	if ($how < 0 || $how > 2)
	$how = 0;
	if ($how == 0) {
		$Mailer = new Mailer(array(
			'Type' => 'PHP'));
	} else if ($how == 1) {
		$sendmail = read_config_option('settings_sendmail_path');
		$Mailer = new Mailer(array(
			'Type' => 'DirectInject',
			'DirectInject_Path' => $sendmail));
	} else if ($how == 2) {
		$smtp_host = read_config_option("settings_smtp_host");
		$smtp_port = read_config_option("settings_smtp_port");
		$smtp_username = read_config_option("settings_smtp_username");
		$smtp_password = read_config_option("settings_smtp_password");

		$Mailer = new Mailer(array(
			'Type' => 'SMTP',
			'SMTP_Host' => $smtp_host,
			'SMTP_Port' => $smtp_port,
			'SMTP_Username' => $smtp_username,
			'SMTP_Password' => $smtp_password));
	}

	if ($from == '') {
		$from = read_config_option('settings_from_email');
		$fromname = read_config_option('settings_from_name');
		if ($from == "") {
			if (isset($_SERVER['HOSTNAME'])) {
				$from = 'Cacti@' . $_SERVER['HOSTNAME'];
			} else {
				$from = 'Cacti@cactiusers.org';
			}
		}
		if ($fromname == "")
		$fromname = "Cacti";

		$from = $Mailer->email_format($fromname, $from);
		if ($Mailer->header_set('From', $from) === false) {
			print "ERROR: " . $Mailer->error() . "\n";
			return $Mailer->error();
		}
	} else {
		$from = $Mailer->email_format('Cacti', $from);
		if ($Mailer->header_set('From', $from) === false) {
			print "ERROR: " . $Mailer->error() . "\n";
			return $Mailer->error();
		}
	}

	if ($to == '')
	return "Mailer Error: No <b>TO</b> address set!!<br>If using the <i>Test Mail</i> link, please set the <b>Alert e-mail</b> setting.";
	$to = explode(',', $to);

	foreach($to as $t) {
		if (trim($t) != '' && !$Mailer->header_set("To", $t)) {
			print "ERROR: " . $Mailer->error() . "\n";
			return $Mailer->error();
		}
	}

	$wordwrap = read_config_option("settings_wordwrap");
	if ($wordwrap == '')
	$wordwrap = 76;
	if ($wordwrap > 9999)
	$wordwrap = 9999;
	if ($wordwrap < 0)
	$wordwrap = 76;

	$Mailer->Config["Mail"]["WordWrap"] = $wordwrap;

	if (! $Mailer->header_set("Subject", $subject)) {
		print "ERROR: " . $Mailer->error() . "\n";
		return $Mailer->error();
	}

	if (is_array($data_array) && !empty($data_array) && strstr($message, '<GRAPH>') !==0) {
		foreach($data_array as $data_item) {
			if ($data_item["attachment"] != "") {
				$cid = $Mailer->content_id();
				if ($Mailer->attach($data_item["attachment"], $data_item['filename'], $data_item["mime_type"], $data_item["inline"], $cid) == false) {
					print "ERROR: " . $Mailer->error() . "\n";
					return $Mailer->error();
				}
				$message = str_replace('<GRAPH>', "<br><br><img src='cid:$cid'>", $message);
			} else {
				$message = str_replace('<GRAPH>', "<br><img src='" . $data_item['file'] . "'><br>Could not open!<br>" . $data_item['file'], $message);
			}
		}
	}
	$text = array('text' => '', 'html' => '');
	if ($data_array == '') {
		$message = str_replace('<br>',  "\n", $message);
		$message = str_replace('<BR>',  "\n", $message);
		$message = str_replace('</BR>', "\n", $message);
		$text['text'] = strip_tags($message);
	} else {
		$text['html'] = $message . '<br>';
		$text['text'] = strip_tags(str_replace('<br>', "\n", $message));
	}

	if ($Mailer->send($text) == false) {
		print "ERROR: " . $Mailer->error() . "\n";
		return $Mailer->error();
	}

	return '';

}


/**
 * convert png images stream to jpeg using php-gd
 *
 * @param unknown_type $png_data	the png image as a stream
 * @return unknown					the jpeg image as a stream
 */
function png2jpeg ($png_data) {
	global $config;

	if ($png_data != "") {
		$fn = "/tmp/" . time() . '.png';

		/* write rrdtool's png file to scratch dir */
		$f = fopen($fn, 'wb');
		fwrite($f, $png_data);
		fclose($f);

		/* create php-gd image object from file */
		$im = imagecreatefrompng($fn);
		if (!$im) {								/* check for errors */
			$im = ImageCreate (150, 30);		/* create an empty image */
			$bgc = ImageColorAllocate ($im, 255, 255, 255);
			$tc  = ImageColorAllocate ($im, 0, 0, 0);
			ImageFilledRectangle ($im, 0, 0, 150, 30, $bgc);
			/* print error message */
			ImageString($im, 1, 5, 5, "Error while opening: $imgname", $tc);
		}

        ob_start(); // start a new output buffer to capture jpeg image stream
		imagejpeg($im);	// output to buffer
		$ImageData = ob_get_contents(); // fetch image from buffer
		$ImageDataLength = ob_get_length();
		ob_end_clean(); // stop this output buffer
		imagedestroy($im); //clean up

		unlink($fn); // delete scratch file
	}
	return $ImageData;
}

/**
 * convert png images stream to gif using php-gd
 *
 * @param unknown_type $png_data	the png image as a stream
 * @return unknown					the gif image as a stream
 */
function png2gif ($png_data) {
	global $config;

	if ($png_data != "") {
		$fn = "/tmp/" . time() . '.png';

		/* write rrdtool's png file to scratch dir */
		$f = fopen($fn, 'wb');
		fwrite($f, $png_data);
		fclose($f);

		/* create php-gd image object from file */
		$im = imagecreatefrompng($fn);
		if (!$im) {								/* check for errors */
			$im = ImageCreate (150, 30);		/* create an empty image */
			$bgc = ImageColorAllocate ($im, 255, 255, 255);
			$tc  = ImageColorAllocate ($im, 0, 0, 0);
			ImageFilledRectangle ($im, 0, 0, 150, 30, $bgc);
			/* print error message */
			ImageString($im, 1, 5, 5, "Error while opening: $imgname", $tc);
		}

        ob_start(); // start a new output buffer to capture gif image stream
		imagegif($im);	// output to buffer
		$ImageData = ob_get_contents(); // fetch image from buffer
		$ImageDataLength = ob_get_length();
		ob_end_clean(); // stop this output buffer
		imagedestroy($im); //clean up

		unlink($fn); // delete scratch file
	}
	return $ImageData;
}

?>
