<?php
/*******************************************************************************

Author ......... Reinhard Scheck
based on work of Jimmy Conner
Contact ........ gandalf@cacti.net
Home Site ......
Program ........ Send Graphs by email
Version ........ 0.1
Purpose ........ Allows you to send graphs via Email

*******************************************************************************/

define("NECTAR_SEND_NOW", 1);
define("NECTAR_DUPLICATE", 2);
define("NECTAR_ENABLE", 3);
define("NECTAR_DISABLE", 4);
define("NECTAR_DELETE", 99);

define("NECTAR_TYPE_PNG", 1);
define("NECTAR_TYPE_JPG", 2);
define("NECTAR_TYPE_GIF", 3);
define("NECTAR_TYPE_PDF", 4);

define("NECTAR_ITEM_GRAPH", 1);
define("NECTAR_ITEM_TEXT", 2);

define("NECTAR_ALIGN_LEFT", 1);
define("NECTAR_ALIGN_CENTER", 2);
define("NECTAR_ALIGN_RIGHT", 3);

define("NECTAR_SCHED_INTVL_DAY", 1);
define("NECTAR_SCHED_INTVL_WEEK", 2);
define("NECTAR_SCHED_INTVL_MONTH_DAY", 3);
define("NECTAR_SCHED_INTVL_MONTH_WEEKDAY", 4);
define("NECTAR_SCHED_INTVL_YEAR", 5);

define("NECTAR_SCHED_COUNT", 1);
define("NECTAR_SCHED_OFFSET", 0);

define("NECTAR_FONT_SIZE", 10);
define("NECTAR_HOST_NONE", 0);
define("NECTAR_TIMESPAN_DEFAULT", GT_LAST_DAY);
define("NECTAR_DEBUG", read_config_option("log_verbosity"), true);
define("NECTAR_EXTENSION_GD", "gd");	# php-gd extension required for png2jpeg, png2gif

function plugin_nectar_install () {

	# graph setup all arrays needed for automation
	api_plugin_register_hook('nectar', 'config_arrays', 'nectar_config_arrays', 'setup.php');
	api_plugin_register_hook('nectar', 'config_form', 'nectar_config_form', 'setup.php');
	api_plugin_register_hook('nectar', 'draw_navigation_text', 'nectar_draw_navigation_text', 'setup.php');
	api_plugin_register_hook('nectar', 'poller_bottom', 'nectar_poller_bottom', 'setup.php');
	api_plugin_register_hook('nectar', 'console_after', 'nectar_console_after', 'setup.php');
	nectar_setup_table ();
}


function plugin_nectar_uninstall () {
	// Do any extra Uninstall stuff here
}

function plugin_nectar_check_config () {
	// Here we will check to ensure everything is configured
	nectar_check_upgrade ();
	return true;
}

function plugin_nectar_upgrade () {
	// Here we will upgrade to the newest version
	nectar_check_upgrade ();
	return true;
}

function plugin_nectar_version () {
	return nectar_version();
}

function nectar_check_upgrade () {
	global $config, $database_default;
	include_once($config["library_path"] . "/database.php");

	// Let's only run this check if we are on a page that actually needs the data
	$files = array('nectar.php,nectar_edit.php');
	if (isset($_SERVER['PHP_SELF']) && !in_array(basename($_SERVER['PHP_SELF']), $files))
	return;

	$current = nectar_version ();
	$current = $current['version'];
	$old = read_config_option('plugin_nectar_version');
	if ($current != $old) {
		// do sth
	}
}

function nectar_check_dependencies() {
	global $plugins, $config;
	return true;
}

/**
 * Version information (used by update plugin)
 */
function nectar_version () {
	return array( 'name' 	=> 'nectar',
				'version' 	=> '0.11',
				'longname'	=> 'Send Graphs via EMail',
				'author'	=> 'Reinhard Scheck',
				'homepage'	=> '',
				'email'		=> 'gandalf@cacti.net',
	#'url'		=> ''
	);
}

function nectar_console_after () {
	nectar_setup_table ();
}

function nectar_config_arrays () {
	global $menu, $messages;
	global $nectar_actions, $attach_types, $item_types, $alignment, $nectar_interval;

	if (function_exists('api_plugin_register_realm')) {
		# register all php modules required for this plugin
		api_plugin_register_realm('nectar', 'nectar.php,nectar_edit.php', 'Send Graphs via EMail', 1);
	}

	$temp = $menu["Utilities"]['logout.php'];
	unset($menu["Utilities"]['logout.php']);
	$menu["Utilities"]['plugins/nectar/nectar.php'] = "Nectar";
	$menu["Utilities"]['logout.php'] = $temp;

	$nectar_actions = array(
	NECTAR_SEND_NOW 	=> "Send Now",
	NECTAR_DUPLICATE => "Duplicate",
	NECTAR_ENABLE 	=> "Enable",
	NECTAR_DISABLE 	=> "Disable",
	NECTAR_DELETE 	=> "Delete",
	);

	$attach_types = array(
	NECTAR_TYPE_PNG => 'Inline PNG Image',
	#NECTAR_TYPE_JPG => 'Inline JPEG Image',
	#NECTAR_TYPE_PDF => 'PDF Attachment',
	);
	if (extension_loaded(NECTAR_EXTENSION_GD)){
		$attach_types[NECTAR_TYPE_JPG] = 'Inline JPEG Image';
		$attach_types[NECTAR_TYPE_GIF] = 'Inline GIF Image';
	}

	$item_types = array(
	NECTAR_ITEM_GRAPH 	=> 'Graph',
	NECTAR_ITEM_TEXT 	=> 'Text',
	);

	$alignment = array(
	NECTAR_ALIGN_LEFT 	=> 'Left',
	NECTAR_ALIGN_CENTER 	=> 'Center',
	NECTAR_ALIGN_RIGHT 	=> 'Right');

	$nectar_interval = array(
	NECTAR_SCHED_INTVL_DAY			=> 'Day(s)',
	NECTAR_SCHED_INTVL_WEEK			=> 'Week(s)',
	NECTAR_SCHED_INTVL_MONTH_DAY	=> 'Month(s), Base: Day',
	NECTAR_SCHED_INTVL_MONTH_WEEKDAY=> 'Month(s), Base: Weekday',
	NECTAR_SCHED_INTVL_YEAR			=> 'Year(s)',
	);

	$messages["mg_mailtime_invalid"]["message"] = "Invalid Timestamp. Select timestamp in the future.";
	$messages["mg_mailtime_invalid"]["type"] = "error";

}


function nectar_config_form () {
	global $config, $graph_timespans, $nectar_interval;
	global $attach_types, $item_types, $alignment;
	global $fields_nectar_edit, $fields_nectar_item_edit;
	#include_once($config["base_path"]."/plugins/nectar/nectar_functions.php");

	# ------------------------------------------------------------
	# Autom8 Rules
	# ------------------------------------------------------------
	/* file: autom8_graph_rules.php, autom8_tree_rules.php, action: edit */
	$fields_nectar_edit = array(
		'name' => array(
				'friendly_name' => 'Report Name',
				'method' => 'textbox',
				'default' => 'New Report',
				'description' => 'Give this Report a descriptive Name',
				'max_length' => 99,
				'value' => '|arg1:name|' ),
		'mailtime' => array(
				'friendly_name' => 'Next Timestamp for sending Mail Report',
				'description' => 'Start time for [first|next] mail to take place. All future mailing times will be based upon this start time. A good example would be 2:00AM.',
				'default' => 0,
				'method' => 'textbox',
				'size' => 20,
				'max_length' => 20,
				'value' => '|arg1:mailtime|' ),
		'intrvl' => array(
				'friendly_name' => 'Report Interval',
				'description' => 'Defines a schedule pattern, relative to given start time, e.g. "Day" or "Month".' . '<br>' .
								'e.g. "Week" for reporting interval with a weekly repeat cycle',
				'default' => NECTAR_SCHED_INTVL_DAY,
				'method' => 'drop_array',
				'array' => $nectar_interval,
				'value' => '|arg1:intrvl|' ),
		'count' => array(
				'friendly_name' => 'Number of Intervals',
				'description' => 'Defines a the number of intervals (see above) to pass until next schedule.' . '<br>' .
								'e.g. "2" for a two week reporting interval',
				'default' => NECTAR_SCHED_COUNT,
				'method' => 'textbox',
				'size' => 20,
				'max_length' => 10,
				'value' => '|arg1:count|' ),
		'offset' => array(
				'friendly_name' => 'Report Offset [seconds]',
				'description' => 'The Offset in Seconds with respect to the (interval, count) base.' . '<br>' .
								'e.g. "43200" for a report that will be offset half a day on each schedule',
				'default' => NECTAR_SCHED_OFFSET,
				'method' => 'textbox',
				'size' => 20,
				'max_length' => 10,
				'value' => '|arg1:offset|' ),
		'from_email' => array(
				'friendly_name' => 'From Email Address',
				'method' => 'textbox',
				'default' => '',
				'description' => 'This Adress will be used as Sender Information',
				'max_length' => 255,
				'value' => '|arg1:from_email|' ),
		'email' => array(
				'friendly_name' => 'To Email Address(es)',
				'method' => 'textbox',
				'default' => '',
				'description' => 'Please seperate multiple adresses by comma (,)',
				'max_length' => 255,
				'value' => '|arg1:email|' ),
		'attachment_type' => array(
				'friendly_name' => 'Image attach type',
				'method' => 'drop_array',
				'default' => NECTAR_TYPE_PNG,
				'description' => 'Select one of the given Types for the Image Attachments',
				'value' => '|arg1:attachment_type|',
				'array' => $attach_types),
		'enabled' => array(
				'friendly_name' => 'Enable Nectar',
				'method' => 'checkbox',
				'default' => '',
				'description' => 'Check this box to enable this Report.',
				'value' => '|arg1:enabled|',
				'form_id' => false),
		'id' => array(
				'method' => 'hidden_zero',
				'value' => '|arg1:id|'),
		'save_component_nectar' => array(
				'method' => 'hidden',
				'value' => '1'),
	);

	$fields_nectar_item_edit = array(
		'item_type' => array(
				'friendly_name' => 'Type',
				'method' => 'drop_array',
				'default' => NECTAR_ITEM_GRAPH,
				'description' => 'Item Type to be added.',
				'value' => '|arg1:item_type|',
				'on_change' => 'toggle_item_type()',
				'array' => $item_types),
		'host_id' => array(
				'friendly_name' => 'Host',
				'method' => 'drop_sql',
				'default' => NECTAR_HOST_NONE,
				'description' => 'Select a Host to specify a Graph',
				'value' => '|arg1:host_id|',
				'on_change' => 'applyHostIdChange(document.nectar_item_edit)',
				'sql' => "(SELECT id, CONCAT_WS('',description,' (',hostname,')') AS name FROM host) UNION (SELECT '" . NECTAR_HOST_NONE . "' AS id, ' NONE' AS name) ORDER BY name"),
		'local_graph_id' => array(
				'friendly_name' => 'Graph',
				'method' => 'drop_sql',
				'default' => 'NULL',
				'description' => 'Graph',
				'value' => '|arg1:local_graph_id|',
				'sql' => "(SELECT graph_templates_graph.local_graph_id as id, graph_templates_graph.title_cache as name
						FROM graph_local LEFT JOIN graph_templates_graph ON (graph_local.id=graph_templates_graph.local_graph_id)
						WHERE graph_local.host_id=|arg1:host_id|)
						UNION (select '" . NECTAR_HOST_NONE . "' as id, ' NONE' as name)
						ORDER BY name"),
		'timespan' => array(
				'friendly_name' => 'Graph Timespan',
				'method' => 'drop_array',
				'default' => GT_LAST_DAY,
				'description' => "Graph End Time is always set to the Nectar's schedule." . '<br>' .
								'Graph Start Time equals Graph End Time minus given timespan',
				'array' => $graph_timespans,
				'value' => '|arg1:timespan|'),
		'align' => array(
				'friendly_name' => 'Alignment',
				'method' => 'drop_array',
				'default' => NECTAR_ALIGN_LEFT,
				'description' => 'Alignment of the Item',
				'value' => '|arg1:align|',
				'array' => $alignment),
		'item_text' => array(
				'friendly_name' => 'Fixed Text',
				'method' => 'textbox',
				'default' => '',
				'description' => 'Enter descriptive Text',
				'max_length' => 255,
				'value' => '|arg1:item_text|'),
		'font_size' => array(
				'friendly_name' => 'Font Size',
				'method' => 'textbox',
				'default' => NECTAR_FONT_SIZE,
				'max_length' => 2,
				'description' => 'Font Size of the Item',
				'value' => '|arg1:font_size|'),
		'sequence' => array(
				'method' => 'view',
				'friendly_name' => 'Sequence',
				'description' => 'Sequence of Item.',
				'value' => '|arg1:sequence|'),
		'report_id' => array(
				'method' => 'hidden_zero',
				'value' => '|arg1:report_id|'),
		'id' => array(
				'method' => 'hidden_zero',
				'value' => '|arg1:id|'),
		'save_component_nectar_item' => array(
				'method' => 'hidden',
				'value' => '1'),
	);
}


function nectar_draw_navigation_text ($nav) {
	$nav["nectar.php:"] 			= array("title" => "Nectar", "mapping" => "index.php:", "url" => "nectar.php", "level" => "1");
	$nav["nectar.php:actions"]		= array("title" => "Report Add", "mapping" => "index.php:", "url" => "nectar.php", "level" => "1");
	$nav["nectar.php:delete"] 		= array("title" => "Report Delete", "mapping" => "index.php:", "url" => "nectar.php", "level" => "1");
	$nav["nectar.php:edit"] 		= array("title" => "Report Edit", "mapping" => "index.php:", "url" => "nectar.php", "level" => "1");
	$nav["nectar.php:item_edit"] 	= array("title" => "Report Edit Item", "mapping" => "index.php:", "url" => "nectar_edit.php", "level" => "1");
	$nav["nectar_edit.php:"] 		= array("title" => "Report Edit", "mapping" => "index.php:", "url" => "nectar.php", "level" => "1");
	$nav["nectar_edit.php:add"] 	= array("title" => "Report Edit Add Item", "mapping" => "index.php:", "url" => "nectar_edit.php", "level" => "1");
	$nav["nectar_edit.php:delete"] 	= array("title" => "Report Edit Delete Item", "mapping" => "index.php:", "url" => "nectar_edit.php", "level" => "1");
	$nav["nectar_edit.php:preview"] = array("title" => "Report Preview", "mapping" => "index.php:", "url" => "nectar_edit.php", "level" => "1");
	return $nav;
}


function nectar_poller_bottom () {
	global $config;
	include_once($config['base_path'] . '/plugins/nectar/nectar_functions.php');

	$t = time();

	# fetch all enabled nectars that have a stratime in the past
	$nectars = db_fetch_assoc("SELECT * from plugin_nectar WHERE mailtime < $t AND enabled='on'");
	nectar_log("nectar_poller_bottom, reports found: " . sizeof($nectars), true, "NECTAR", POLLER_VERBOSITY_LOW);

	# execute each of those nectars
	foreach ($nectars as $nectar) {
		if (isset($nectar['email'])) {
			generate_nectar($nectar, false);
		}
	}
}


function nectar_setup_table () {
	global $config, $database_default;
	include_once($config["library_path"] . "/database.php");

	/* are my tables already present? */
	$sql	= "show tables from `" . $database_default . "`";
	$result = db_fetch_assoc($sql) or die (mysql_error());
	$tables = array();
	$sql 	= array();

	foreach($result as $index => $arr) {
		foreach($arr as $tbl) {
			$tables[] = $tbl;
		}
	}

	if (!in_array('plugin_nectar', $tables)) {
		$data = array();
		$data['columns'][] = array('name' => 'id', 				'type' => 'mediumint(8)', 	'unsigned' => 'unsigned', 	'NULL' => false, 'auto_increment' => true);
		$data['columns'][] = array('name' => 'name',			'type' => 'varchar(100)',          		                'NULL' => false, 'default' => '');
		$data['columns'][] = array('name' => 'intrvl',	 		'type' => 'smallint(2)',  	'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'count',	 		'type' => 'smallint(2)',  	'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'offset',	 		'type' => 'int(12)',  		'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'mailtime',	 	'type' => 'bigint',    	  	'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'from_email', 		'type' => 'text',  										'NULL' => false,);
		$data['columns'][] = array('name' => 'email',	 		'type' => 'text',  	             			           	'NULL' => false,);
		$data['columns'][] = array('name' => 'attachment_type', 'type' => 'smallint(2)',  	'unsigned' => 'unsigned',	'NULL' => false, 'default' => NECTAR_TYPE_PNG);
		$data['columns'][] = array('name' => 'lastsent',	 	'type' => 'bigint', 		'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'enabled', 		'type' => 'char(2)',									'NULL' => true,  'default' => '');
		$data['primary'] = 'id';
		$data['keys'][] = array('name' => 'mailtime',	'columns' => 'mailtime');
		$data['type'] = 'MyISAM';
		$data['comment'] = 'Nectar Reports';
		api_plugin_db_table_create ('nectar', 'plugin_nectar', $data);
	}

	if (!in_array('plugin_nectar_items', $tables)) {
		$data = array();
		$data['columns'][] = array('name' => 'id', 				'type' => 'int(10)', 	'unsigned' => 'unsigned', 	'NULL' => false, 'auto_increment' => true);
		$data['columns'][] = array('name' => 'report_id',		'type' => 'int(10)',    'unsigned' => 'unsigned',   'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'item_type', 		'type' => 'tinyint(1)', 'unsigned' => 'unsigned', 	'NULL' => false, 'default' => NECTAR_ITEM_GRAPH);
		$data['columns'][] = array('name' => 'host_id',			'type' => 'int(10)',  	'unsigned' => 'unsigned', 	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'local_graph_id',	'type' => 'int(10)',  	'unsigned' => 'unsigned', 	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'timespan',	 	'type' => 'int',    	'unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['columns'][] = array('name' => 'align',	 		'type' => 'tinyint(1)', 'unsigned' => 'unsigned', 	'NULL' => false, 'default' => NECTAR_ALIGN_LEFT);
		$data['columns'][] = array('name' => 'item_text',		'type' => 'text',                           		'NULL' => false,);
		$data['columns'][] = array('name' => 'font_size', 		'type' => 'smallint(2)','unsigned' => 'unsigned', 	'NULL' => false, 'default' => NECTAR_FONT_SIZE);
		$data['columns'][] = array('name' => 'sequence', 		'type' => 'smallint(5)','unsigned' => 'unsigned',	'NULL' => false, 'default' => 0);
		$data['primary'] = 'id';
		$data['keys'][] = array('name' => 'report_id',	'columns' => 'report_id');
		$data['type'] = 'MyISAM';
		$data['comment'] = 'Nectar Report Items';
		api_plugin_db_table_create ('nectar', 'plugin_nectar_items', $data);

		# now run all SQL commands
		#if (!empty($sql)) {
		#	for ($a = 0; $a < count($sql); $a++) {
		#		$result = db_execute($sql[$a]);
		#	}
		#}
	}
}

/**
 * PHP error handler
 * @arg $errno
 * @arg $errmsg
 * @arg $filename
 * @arg $linenum
 * @arg $vars
 */
function nectar_error_handler($errno, $errmsg, $filename, $linenum, $vars) {

	$errno = $errno & error_reporting();
	# return if error handling disabled by @
	if($errno == 0) return;
	# define constants not available with PHP 4
	if(!defined('E_STRICT'))            define('E_STRICT', 2048);
	if(!defined('E_RECOVERABLE_ERROR')) define('E_RECOVERABLE_ERROR', 4096);

	if (read_config_option("log_verbosity") >= POLLER_VERBOSITY_HIGH) {
		/* define all error types */
		$errortype = array(
		E_ERROR             => 'Error',
		E_WARNING           => 'Warning',
		E_PARSE             => 'Parsing Error',
		E_NOTICE            => 'Notice',
		E_CORE_ERROR        => 'Core Error',
		E_CORE_WARNING      => 'Core Warning',
		E_COMPILE_ERROR     => 'Compile Error',
		E_COMPILE_WARNING   => 'Compile Warning',
		E_USER_ERROR        => 'User Error',
		E_USER_WARNING      => 'User Warning',
		E_USER_NOTICE       => 'User Notice',
		E_STRICT            => 'Runtime Notice',
		E_RECOVERABLE_ERROR => 'Catchable Fatal Error'
		);

		/* create an error string for the log */
		$err = "ERRNO:'"  . $errno   . "' TYPE:'"    . $errortype[$errno] .
			"' MESSAGE:'" . $errmsg  . "' IN FILE:'" . $filename .
			"' LINE NO:'" . $linenum . "'";

		/* let's ignore some lesser issues */
		if (substr_count($errmsg, "date_default_timezone")) return;
		if (substr_count($errmsg, "Only variables")) return;

		/* log the error to the Cacti log */
		#autom8_log("PROGERR: " . $err, false, "AUTOM8");
		print("PROGERR: " . $err . "<br><pre>");# print_r($vars); print("</pre>");

		# backtrace, if available
		if (function_exists('debug_backtrace')) {
			//print "backtrace:\n";
			$backtrace = debug_backtrace();
			array_shift($backtrace);
			foreach($backtrace as $i=>$l) {
				print "[$i] in function <b>{$l['class']}{$l['type']}{$l['function']}</b>";
				if($l['file']) print " in <b>{$l['file']}</b>";
				if($l['line']) print " on line <b>{$l['line']}</b>";
				print "\n";
			}
		}
		if (isset($GLOBALS['error_fatal'])) {
			if($GLOBALS['error_fatal'] & $errno) die('fatal');
		}
	}

	return;
}

